@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -42,7 +42,7 @@
 
 static REHex::ToolPanel *BitmapTool_factory(wxWindow *parent, REHex::SharedDocumentPointer &document, REHex::DocumentCtrl *document_ctrl)
 {
-	return new REHex::BitmapTool(parent, document);
+	return new REHex::BitmapTool(parent, document, document_ctrl);
 }
 
 static REHex::ToolPanelRegistration tpr("BitmapTool", "Bitmap visualisation", REHex::ToolPanel::TPS_TALL, &BitmapTool_factory);
@@ -135,9 +135,10 @@ static const int ZOOM_LEVELS[] = {
 
 static const int LAST_ZOOM_LEVEL_IDX = sizeof(ZOOM_LEVELS) / sizeof(*ZOOM_LEVELS) - 1;
 
-REHex::BitmapTool::BitmapTool(wxWindow *parent, SharedDocumentPointer &document):
+REHex::BitmapTool::BitmapTool(wxWindow *parent, SharedDocumentPointer &document, DocumentCtrl *document_ctrl):
 	ToolPanel(parent),
 	document(document),
+	document_ctrl(document_ctrl),
 	image_offset(document->get_cursor_position()),
 	image_width(256),
 	image_height(256),
@@ -167,7 +168,8 @@ REHex::BitmapTool::BitmapTool(wxWindow *parent, SharedDocumentPointer &document)
 	wxSize text_size = offset_textctrl->GetTextExtent("0x0000000000000000+0b");
 	offset_textctrl->SetMinSize(wxSize(((float)(text_size.GetWidth()) * 1.2f), initial_size.GetHeight()));
 	
-	offset_textctrl->ChangeValue(format_offset(image_offset, OFFSET_BASE_DEC, BitOffset::ZERO));
+	OffsetBase base = document_ctrl->get_offset_display_base();
+	offset_textctrl->ChangeValue(image_offset.to_string((base == OFFSET_BASE_HEX ? NumBase::HEX : NumBase::DEC), NumFormat::PREFIX));
 	offset_follow_cb->SetValue(true);
 	
 	sizer_add_pair("Image width:",  (width_textctrl = new wxSpinCtrl(this, ID_IMAGE_WIDTH, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 10240, image_width)));
@@ -277,13 +279,25 @@ REHex::BitmapTool::BitmapTool(wxWindow *parent, SharedDocumentPointer &document)
 	update();
 }
 
-REHex::BitmapTool::~BitmapTool() {}
+REHex::BitmapTool::~BitmapTool() {
+	delete bitmap;
+}
 
 std::string REHex::BitmapTool::name() const
 {
 	return "BitmapTool";
 }
 
+std::string REHex::BitmapTool::label() const
+{
+	return "Bitmap visualisation";
+}
+
+REHex::ToolPanel::Shape REHex::BitmapTool::shape() const
+{
+	return ToolPanel::TPS_TALL;
+}
+
 void REHex::BitmapTool::save_state(wxConfig *config) const
 {
 	// TODO
@@ -832,7 +846,7 @@ void REHex::BitmapTool::render_region(int region_y, int region_h, BitOffset offs
 		}
 		
 		assert((line_off - data_begin).byte_aligned());
-		const unsigned char *line_ptr = data.data() + (line_off - data_begin).byte();
+		off_t data_line_offset = (line_off - data_begin).byte();
 		
 		wxNativePixelData::Iterator output_col_ptr = output_ptr;
 		
@@ -853,7 +867,23 @@ void REHex::BitmapTool::render_region(int region_y, int region_h, BitOffset offs
 				input_x = ((width - 1) - input_x);
 			}
 			
-			const unsigned char *input_ptr = line_ptr;
+			/* Initialise output to chequerboard pattern. */
+			
+			bool x_even = (output_x % 20) >= 10;
+			bool y_even = (output_y % 20) >= 10;
+			
+			int chequerboard_colour = (x_even ^ y_even) ? 0x66 : 0x99;
+			
+			output_col_ptr.Red()   = chequerboard_colour;
+			output_col_ptr.Green() = chequerboard_colour;
+			output_col_ptr.Blue()  = chequerboard_colour;
+			
+			if((off_t)(data.size()) <= data_line_offset)
+			{
+				continue;
+			}
+			
+			const unsigned char *input_ptr = data.data() + data_line_offset;
 			int mask = pixel_fmt_bits, shift = 8 - (8 / pixel_fmt_div);
 			
 			if(pixel_fmt_div > 1)
@@ -886,17 +916,6 @@ void REHex::BitmapTool::render_region(int region_y, int region_h, BitOffset offs
 				input_ptr += input_x * pixel_fmt_multi;
 			}
 			
-			/* Initialise output to chequerboard pattern. */
-			
-			bool x_even = (output_x % 20) >= 10;
-			bool y_even = (output_y % 20) >= 10;
-			
-			int chequerboard_colour = (x_even ^ y_even) ? 0x66 : 0x99;
-			
-			output_col_ptr.Red()   = chequerboard_colour;
-			output_col_ptr.Green() = chequerboard_colour;
-			output_col_ptr.Blue()  = chequerboard_colour;
-			
 			if((input_ptr + pixel_fmt_multi) > (data.data() + data.size()))
 			{
 				/* Ran out of image data in input file. Carry on looping to fill
@@ -963,7 +982,8 @@ void REHex::BitmapTool::OnCursorUpdate(CursorUpdateEvent &event)
 {
 	if(offset_follow_cb->GetValue())
 	{
-		offset_textctrl->ChangeValue(format_offset(event.cursor_pos, OFFSET_BASE_DEC, BitOffset::ZERO));
+		OffsetBase base = document_ctrl->get_offset_display_base();
+		offset_textctrl->ChangeValue(event.cursor_pos.to_string((base == OFFSET_BASE_HEX ? NumBase::HEX : NumBase::DEC), NumFormat::PREFIX));
 		update();
 	}
 	
@@ -989,7 +1009,8 @@ void REHex::BitmapTool::OnFollowCursor(wxCommandEvent &event)
 {
 	if(offset_follow_cb->GetValue())
 	{
-		offset_textctrl->ChangeValue(format_offset(document->get_cursor_position(), OFFSET_BASE_DEC, BitOffset::ZERO));
+		OffsetBase base = document_ctrl->get_offset_display_base();
+		offset_textctrl->ChangeValue(document->get_cursor_position().to_string((base == OFFSET_BASE_HEX ? NumBase::HEX : NumBase::DEC), NumFormat::PREFIX));
 		update();
 	}
 	else{
