@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -30,6 +30,9 @@
 #include <vector>
 
 #include "BitOffset.hpp"
+#include "ByteRangeSet.hpp"
+#include "profile.hpp"
+#include "shared_mutex.hpp"
 
 namespace REHex
 {
@@ -101,6 +104,7 @@ namespace REHex
 			 * scratch when a nearby offset is requested again.
 			*/
 			mutable typename std::vector< std::pair<Range, T> >::const_iterator last_get_iter;
+			mutable shared_mutex lgi_mutex;
 			
 		public:
 			/**
@@ -169,6 +173,18 @@ namespace REHex
 			*/
 			void set_range(OT offset, OT length, const T &value);
 			
+			/**
+			 * @brief Set multiple ranges of bytes in the map at once.
+			 *
+			 * This method provides an optimised version of calling set_range()
+			 * multiple times. Each element in the vector contains the values that
+			 * would be passed to the set_range() method.
+			 *
+			 * If any ranges overlap, the later ones in the list take priority as if
+			 * calling the set_range() method individually.
+			*/
+			void set_bulk(std::vector< std::pair<Range, T> > &&bulk_ranges);
+			
 			/**
 			 * @brief Clear a range of bytes in the map.
 			 *
@@ -226,6 +242,8 @@ namespace REHex
 		private:
 			bool data_inserted_impl(OT offset, OT length);
 			bool data_erased_impl(OT offset, OT length);
+
+			static bool elem_key_less(const std::pair<Range, T> &a, const std::pair<Range, T> &b);
 			
 		public:
 			/**
@@ -283,13 +301,17 @@ namespace REHex
 
 template<typename OT, typename T> typename REHex::RangeMap<OT, T>::const_iterator REHex::RangeMap<OT, T>::get_range(OT offset) const
 {
-	if(last_get_iter != ranges.end() && last_get_iter->first.offset <= offset && (last_get_iter->first.offset + last_get_iter->first.length) > offset)
 	{
-		return last_get_iter;
+		shared_lock lock_guard(lgi_mutex);
+		
+		if(last_get_iter != ranges.end() && last_get_iter->first.offset <= offset && (last_get_iter->first.offset + last_get_iter->first.length) > offset)
+		{
+			return last_get_iter;
+		}
 	}
 	
 	/* Starting from the first element after us (or the end of the vector)... */
-	auto i = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + 1), 0), default_value));
+	auto i = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + 1), 0), default_value), &elem_key_less);
 	
 	/* ...check to see if there is an element prior... */
 	if(i != ranges.begin())
@@ -300,7 +322,12 @@ template<typename OT, typename T> typename REHex::RangeMap<OT, T>::const_iterato
 		if(i->first.offset <= offset && (i->first.offset + i->first.length) > offset)
 		{
 			/* ...it does, return it. */
-			last_get_iter = i;
+			
+			{
+				std::unique_lock<shared_mutex> lock_guard(lgi_mutex);
+				last_get_iter = i;
+			}
+			
 			return i;
 		}
 	}
@@ -316,7 +343,7 @@ template<typename OT, typename T> typename REHex::RangeMap<OT, T>::const_iterato
 		return ranges.end();
 	}
 	
-	auto i = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range(offset, 0), default_value));
+	auto i = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range(offset, 0), default_value), &elem_key_less);
 	
 	if(i != ranges.begin())
 	{
@@ -343,6 +370,8 @@ template<typename OT, typename T> typename REHex::RangeMap<OT, T>::const_iterato
 
 template<typename OT, typename T> void REHex::RangeMap<OT, T>::set_range(OT offset, OT length, const T &value)
 {
+	PROFILE_BLOCK("REHex::RangeMap::set_range()");
+	
 	if(length <= 0)
 	{
 		return;
@@ -353,7 +382,7 @@ template<typename OT, typename T> void REHex::RangeMap<OT, T>::set_range(OT offs
 	*/
 	
 	/* Starting from the first element after us (or the end of the vector)... */
-	auto next = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + length), 0), default_value));
+	auto next = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + length), 0), default_value), &elem_key_less);
 	
 	typename std::vector< std::pair<Range, T> >::iterator erase_begin = next;
 	typename std::vector< std::pair<Range, T> >::iterator erase_end   = next;
@@ -366,6 +395,15 @@ template<typename OT, typename T> void REHex::RangeMap<OT, T>::set_range(OT offs
 		/* ...walking backwards... */
 		auto eb_prev = std::prev(erase_begin);
 		
+		if((eb_prev->first.offset + eb_prev->first.length) == offset && eb_prev->second != value)
+		{
+			/* ...the previous element is adjacent but has a different value, we can
+			 *    leave it untouched.
+			*/
+			
+			break;
+		}
+		
 		if((eb_prev->first.offset + eb_prev->first.length) >= offset)
 		{
 			/* ...the previous element intersects the range we wish to set to some extent... */
@@ -430,38 +468,126 @@ template<typename OT, typename T> void REHex::RangeMap<OT, T>::set_range(OT offs
 		++erase_end;
 	}
 	
+	if((erase_end - erase_begin) == 1 && insert_before.empty() && insert_after.empty())
+	{
+		/* Fast path: overwrite element in-place when we are replacing a single one. */
+		
+		erase_begin->first.offset = offset;
+		erase_begin->first.length = length;
+		erase_begin->second = value;
+		
+		last_get_iter = ranges.end();
+		
+		return;
+	}
+	
 	/* Erase adjacent and/or overlapping ranges. */
 	erase_end = ranges.erase(erase_begin, erase_end);
 	
 	assert(length > 0);
 	
-	if(!insert_before.empty())
+	/* Insert the new range and old intersecting ranges (if applicable). */
+	if(insert_before.empty() && insert_after.empty())
 	{
-		/* Re-insert range with different value immediately before the range beging set
-		 * that was lost above.
-		*/
-		
-		erase_end = ranges.insert(erase_end, insert_before.front());
-		++erase_end;
+		ranges.emplace(erase_end, Range(offset, length), value);
 	}
-	
-	/* Insert new range. */
-	erase_end = ranges.insert(erase_end, std::make_pair(Range(offset, length), value));
-	++erase_end;
-	
-	if(!insert_after.empty())
+	else if(insert_after.empty())
 	{
-		/* Re-insert range with different value immediately after the range beging set
-		 * that was lost above.
-		*/
-		
-		erase_end = ranges.insert(erase_end, insert_after.front());
-		++erase_end;
+		ranges.insert(erase_end, { insert_before.front(), std::make_pair(Range(offset, length), value) });
+	}
+	else if(insert_before.empty())
+	{
+		ranges.insert(erase_end, { std::make_pair(Range(offset, length), value), insert_after.front() });
+	}
+	else{
+		ranges.insert(erase_end, { insert_before.front(), std::make_pair(Range(offset, length), value), insert_after.front() });
 	}
 	
 	last_get_iter = ranges.end();
 }
 
+template<typename OT, typename T> void REHex::RangeMap<OT, T>::set_bulk(std::vector< std::pair<Range, T> > &&bulk_ranges)
+{
+	/* Work backwards from the end of bulk_ranges, removing any earlier overlaps. */
+	
+	RangeSet<OT> seen_ranges;
+	for(auto it = bulk_ranges.end(); it != bulk_ranges.begin();)
+	{
+		--it;
+		
+		OT range_off = it->first.offset;
+		OT range_len = it->first.length;
+		
+		auto collision = seen_ranges.find_last_in(range_off, range_len);
+		if(collision != seen_ranges.end())
+		{
+			OT collision_end = collision->offset + collision->length;
+			OT range_end = range_off + range_len;
+			
+			if(range_end > collision_end && range_off < collision->offset)
+			{
+				/* Collision is in the middle of the range to be set.
+				 * Split this range in two around the collision.
+				*/
+				
+				it = bulk_ranges.emplace(it, Range(range_off, (collision->offset - range_off)), it->second);
+				++it;
+				
+				it->first.offset = collision_end;
+				it->first.length = range_end - collision_end;
+			}
+			else if(range_end > collision_end)
+			{
+				/* Collision is from the beginning of the range.
+				 * Move the range up to the end of the collision.
+				*/
+				
+				it->first.offset = collision_end;
+				it->first.length = range_end - collision_end;
+			}
+			else if(range_off < collision->offset)
+			{
+				/* Collision is to the end of the range.
+				 * Truncate the range up to the collision.
+				*/
+				
+				it->first.length = collision->offset - range_off;
+			}
+			else{
+				/* Collision entirely encompasses the range.
+				 * Discard the range.
+				*/
+				
+				it = bulk_ranges.erase(it);
+				continue;
+			}
+		}
+		else{
+			/* No collision found. */
+		}
+		
+		assert(!(seen_ranges.isset_any(it->first.offset, it->first.length)));
+		seen_ranges.set_range(it->first.offset, it->first.length);
+	}
+	
+	/* Sort bulk_ranges by its elements offsets. There should be no overlaps at this point. */
+	
+	std::sort(bulk_ranges.begin(), bulk_ranges.end(), [](const std::pair<Range, T> &a, const std::pair<Range, T> &b)
+	{
+		return a.first.offset < b.first.offset;
+	});
+	
+	/* Call set_range() for each range in order. There are further optimisations that could
+	 * probably be made here, but just ensuring elements are inserted sequentially already nets
+	 * a significant performance boost when populating large sets.
+	*/
+	
+	for(auto it = bulk_ranges.begin(); it != bulk_ranges.end(); ++it)
+	{
+		set_range(it->first.offset, it->first.length, it->second);
+	}
+}
+
 template<typename OT, typename T> void REHex::RangeMap<OT, T>::clear_range(OT offset, OT length)
 {
 	if(length <= 0)
@@ -474,7 +600,7 @@ template<typename OT, typename T> void REHex::RangeMap<OT, T>::clear_range(OT of
 	*/
 	
 	/* Starting from the first element after us (or the end of the vector)... */
-	auto next = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + length), 0), default_value));
+	auto next = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + length), 0), default_value), &elem_key_less);
 	
 	typename std::vector< std::pair<Range, T> >::iterator erase_begin = next;
 	typename std::vector< std::pair<Range, T> >::iterator erase_end   = next;
@@ -672,7 +798,7 @@ template<typename OT, typename T> bool REHex::RangeMap<OT, T>::data_erased_impl(
 {
 	/* Find the range of elements overlapping the range to be erased. */
 	
-	auto next = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + length), 0), default_value));
+	auto next = std::lower_bound(ranges.begin(), ranges.end(), std::make_pair(Range((offset + length), 0), default_value), &elem_key_less);
 	
 	typename std::vector< std::pair<Range, T> >::iterator erase_begin = next;
 	typename std::vector< std::pair<Range, T> >::iterator erase_end   = next;
@@ -755,4 +881,9 @@ template<typename OT, typename T> bool REHex::RangeMap<OT, T>::data_erased_impl(
 	return elements_changed;
 }
 
+template<typename OT, typename T> bool REHex::RangeMap<OT, T>::elem_key_less(const std::pair<Range, T> &a, const std::pair<Range, T> &b)
+{
+	return a.first < b.first;
+}
+
 #endif /* !REHEX_BYTERANGEMAP_HPP */
