@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -126,9 +126,6 @@ namespace REHex
 			*/
 			RangeSet() {}
 			
-			RangeSet(const RangeSet<OT> &src):
-				ranges(src.ranges) {}
-			
 			/**
 			 * @brief Construct a set from a sequence of ranges.
 			 *
