@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2022-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2022-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -385,6 +385,16 @@ std::string REHex::DataHistogramPanel::name() const
 	return "DataHistogramPanel";
 }
 
+std::string REHex::DataHistogramPanel::label() const
+{
+	return "Histogram";
+}
+
+REHex::ToolPanel::Shape REHex::DataHistogramPanel::shape() const
+{
+	return ToolPanel::TPS_TALL;
+}
+
 void REHex::DataHistogramPanel::save_state(wxConfig *config) const
 {}
 
