@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -104,6 +104,8 @@ void REHex::DisassemblyRegion::OnDataOverwrite(OffsetLengthEvent &event)
 
 int REHex::DisassemblyRegion::calc_width(DocumentCtrl &doc_ctrl)
 {
+	const FontCharacterCache &fcc = doc_ctrl.get_fcc();
+	
 	int indent_width = doc_ctrl.indent_width(indent_depth);
 	
 	int offset_column_width = doc_ctrl.get_show_offsets()
@@ -121,14 +123,14 @@ int REHex::DisassemblyRegion::calc_width(DocumentCtrl &doc_ctrl)
 	offset_text_x = indent_width;
 	hex_text_x    = offset_text_x + offset_column_width;
 	code_text_x   = hex_text_x
-		+ doc_ctrl.hf_string_width(
+		+ fcc.fixed_string_width(
 			(bytes_per_line * 2)
 			+ ((bytes_per_line - 1) / bytes_per_group)
 			+ 1);
 	ascii_text_x = code_text_x
-		+ doc_ctrl.hf_string_width(longest_disasm + 1);
+		+ fcc.fixed_string_width(longest_disasm + 1);
 	
-	return ascii_text_x + doc_ctrl.hf_string_width(ascii_column_chars) + indent_width;
+	return ascii_text_x + fcc.fixed_string_width(ascii_column_chars) + indent_width;
 }
 
 void REHex::DisassemblyRegion::calc_height(DocumentCtrl &doc_ctrl)
@@ -146,6 +148,8 @@ void REHex::DisassemblyRegion::calc_height(DocumentCtrl &doc_ctrl)
 
 void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int64_t y)
 {
+	const FontCharacterCache &fcc = doc_ctrl.get_fcc();
+	
 	if(arch == CS_ARCH_X86)
 	{
 		AsmSyntax new_preferred_asm_syntax = wxGetApp().settings->get_preferred_asm_syntax();
@@ -176,8 +180,8 @@ void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int
 	
 	draw_container(doc_ctrl, dc, x, y);
 	
-	int hf_char_height = doc_ctrl.hf_char_height();
-	int hf_char_width = doc_ctrl.hf_char_width();
+	int hf_char_height = fcc.fixed_char_height();
+	int hf_char_width = fcc.fixed_char_width();
 	
 	if(doc_ctrl.get_show_offsets())
 	{
@@ -235,7 +239,7 @@ void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int
 		(*active_palette)[Palette::PAL_SELECTED_TEXT_FG],
 		(doc_ctrl.hex_view_active()
 			? (*active_palette)[Palette::PAL_SELECTED_TEXT_BG]
-			: active_palette->get_average_colour(Palette::PAL_SELECTED_TEXT_BG, Palette::PAL_NORMAL_TEXT_BG)));
+			: active_palette->blend_colours(Palette::PAL_SELECTED_TEXT_BG, Palette::PAL_NORMAL_TEXT_BG)));
 	
 	auto hex_highlight_func = [&](BitOffset offset)
 	{
@@ -252,7 +256,7 @@ void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int
 		(*active_palette)[Palette::PAL_SELECTED_TEXT_FG],
 		(doc_ctrl.ascii_view_active()
 			? (*active_palette)[Palette::PAL_SELECTED_TEXT_BG]
-			: active_palette->get_average_colour(Palette::PAL_SELECTED_TEXT_BG, Palette::PAL_NORMAL_TEXT_BG)));
+			: active_palette->blend_colours(Palette::PAL_SELECTED_TEXT_BG, Palette::PAL_NORMAL_TEXT_BG)));
 	
 	auto ascii_highlight_func = [&](BitOffset offset)
 	{
@@ -280,7 +284,7 @@ void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int
 		{
 			wxColour selected_bg_colour = doc_ctrl.special_view_active()
 				? (*active_palette)[Palette::PAL_SELECTED_TEXT_BG]
-				: active_palette->get_average_colour(Palette::PAL_SELECTED_TEXT_BG, Palette::PAL_NORMAL_TEXT_BG);
+				: active_palette->blend_colours(Palette::PAL_SELECTED_TEXT_BG, Palette::PAL_NORMAL_TEXT_BG);
 			
 			dc.SetTextForeground((*active_palette)[Palette::PAL_SELECTED_TEXT_FG]);
 			dc.SetTextBackground(selected_bg_colour);
@@ -298,6 +302,8 @@ void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int
 	const std::vector<Instruction> *instr_vec = &(instr_first.first);
 	std::vector<Instruction>::const_iterator instr = instr_first.second;
 	
+	bool has_focus = doc_ctrl.HasFocus();
+	
 	bool is_last_data_region = (doc_ctrl.get_data_regions().back() == this);
 	
 	while(instr != instr_vec->end() && y < client_size.GetHeight() && line_num < (y_lines - indent_final))
@@ -319,11 +325,11 @@ void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int
 			dc.DrawText(offset_str, x + offset_text_x, y);
 		}
 		
-		draw_hex_line(&doc_ctrl, dc, x + hex_text_x, y, instr->data.data(), instr->length, 0, abs_inst_offset, alternate, hex_highlight_func, is_last_line);
+		draw_hex_line(&doc_ctrl, dc, x + hex_text_x, y, instr->data.data(), instr->length, 0, abs_inst_offset, alternate, has_focus, doc_ctrl.hex_view_active(), hex_highlight_func, is_last_line);
 		
 		if(doc_ctrl.get_show_ascii())
 		{
-			draw_ascii_line(&doc_ctrl, dc, x + ascii_text_x, y, instr->data.data(), instr->length, 0, 0, 0, abs_inst_offset, alternate, ascii_highlight_func, is_last_line);
+			draw_ascii_line(&doc_ctrl, dc, x + ascii_text_x, y, instr->data.data(), instr->length, 0, 0, 0, abs_inst_offset, alternate, has_focus, doc_ctrl.ascii_view_active(), ascii_highlight_func, is_last_line);
 		}
 		
 		bool invert = cursor_pos >= abs_inst_offset && cursor_pos < (abs_inst_offset + BitOffset::BYTES(instr->length)) && doc_ctrl.get_cursor_visible() && doc_ctrl.special_view_active();
@@ -399,11 +405,11 @@ void REHex::DisassemblyRegion::draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int
 		const unsigned char *ldp = data_err ? NULL : line_data.data();
 		size_t ldl = data_err ? line_len : line_data.size();
 		
-		draw_hex_line(&doc_ctrl, dc, x + hex_text_x, y, ldp, ldl, 0, abs_up_off, alternate, hex_highlight_func, is_last_line);
+		draw_hex_line(&doc_ctrl, dc, x + hex_text_x, y, ldp, ldl, 0, abs_up_off, alternate, has_focus, doc_ctrl.hex_view_active(), hex_highlight_func, is_last_line);
 		
 		if(doc_ctrl.get_show_ascii())
 		{
-			draw_ascii_line(&doc_ctrl, dc, x + ascii_text_x, y, ldp, ldl, 0, 0, 0, abs_up_off, alternate, ascii_highlight_func, is_last_line);
+			draw_ascii_line(&doc_ctrl, dc, x + ascii_text_x, y, ldp, ldl, 0, 0, 0, abs_up_off, alternate, has_focus, doc_ctrl.ascii_view_active(), ascii_highlight_func, is_last_line);
 		}
 		
 		set_text_attribs(false, false);
@@ -521,6 +527,8 @@ unsigned int REHex::DisassemblyRegion::check()
 
 std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea> REHex::DisassemblyRegion::offset_at_xy(DocumentCtrl &doc_ctrl, int mouse_x_px, int64_t mouse_y_lines)
 {
+	const FontCharacterCache &fcc = doc_ctrl.get_fcc();
+	
 	int64_t processed_lines = this->processed_lines();
 	
 	if(mouse_y_lines < processed_lines)
@@ -540,7 +548,7 @@ std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea>
 		{
 			/* Mouse in ASCII area. */
 			
-			unsigned int line_offset = doc_ctrl.hf_char_at_x(mouse_x_px - ascii_text_x);
+			unsigned int line_offset = fcc.fixed_char_at_x(mouse_x_px - ascii_text_x);
 			if(line_offset >= instr.second->length)
 			{
 				return std::make_pair(BitOffset::INVALID, SA_NONE);
@@ -552,7 +560,7 @@ std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea>
 		{
 			/* Mouse in code area. */
 			
-			unsigned int char_offset = doc_ctrl.hf_char_at_x(mouse_x_px - code_text_x);
+			unsigned int char_offset = fcc.fixed_char_at_x(mouse_x_px - code_text_x);
 			if(char_offset < instr.second->disasm.length())
 			{
 				return std::make_pair(abs_inst_offset, SA_SPECIAL);
@@ -590,7 +598,7 @@ std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea>
 		{
 			/* Mouse in ASCII area. */
 			
-			unsigned int line_offset = doc_ctrl.hf_char_at_x(mouse_x_px - ascii_text_x);
+			unsigned int line_offset = fcc.fixed_char_at_x(mouse_x_px - ascii_text_x);
 			if(line_offset >= line_len)
 			{
 				return std::make_pair(BitOffset::INVALID, SA_NONE);
@@ -620,6 +628,8 @@ std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea>
 
 std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea> REHex::DisassemblyRegion::offset_near_xy(DocumentCtrl &doc_ctrl, int mouse_x_px, int64_t mouse_y_lines, ScreenArea type_hint)
 {
+	const FontCharacterCache &fcc = doc_ctrl.get_fcc();
+	
 	int64_t processed_lines = this->processed_lines();
 	
 	if(mouse_y_lines < processed_lines)
@@ -645,7 +655,7 @@ std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea>
 				return std::make_pair(d_offset + BitOffset(std::max<off_t>((instr_base - 1), 0), 0), SA_ASCII);
 			}
 			else{
-				unsigned int line_offset = doc_ctrl.hf_char_at_x(mouse_x_px - ascii_text_x);
+				unsigned int line_offset = fcc.fixed_char_at_x(mouse_x_px - ascii_text_x);
 				
 				off_t real_offset = std::min(
 					(instr_base + line_offset),
@@ -663,7 +673,7 @@ std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea>
 				return std::make_pair(BitOffset(std::max<off_t>((instr.second->offset - 1), 0), 0), SA_SPECIAL);
 			}
 			
-			unsigned int char_offset = doc_ctrl.hf_char_at_x(mouse_x_px - code_text_x);
+			unsigned int char_offset = fcc.fixed_char_at_x(mouse_x_px - code_text_x);
 			if(char_offset < instr.second->disasm.length())
 			{
 				return std::make_pair(d_offset + BitOffset(instr.second->offset, 0), SA_SPECIAL);
@@ -716,7 +726,7 @@ std::pair<REHex::BitOffset, REHex::DocumentCtrl::GenericDataRegion::ScreenArea>
 				return std::make_pair(d_offset + BitOffset(std::max<off_t>((line_base - 1), 0), 0), SA_ASCII);
 			}
 			else{
-				unsigned int line_offset = doc_ctrl.hf_char_at_x(mouse_x_px - ascii_text_x);
+				unsigned int line_offset = fcc.fixed_char_at_x(mouse_x_px - ascii_text_x);
 				
 				off_t real_offset = std::min(
 					(line_base + line_offset),
@@ -1129,6 +1139,8 @@ REHex::BitOffset REHex::DisassemblyRegion::nth_row_nearest_column(int64_t row, i
 		off_t up_base = unprocessed_offset_rel();
 		int64_t up_row = row - processed_lines;
 		
+		column = std::min<int>(column, max_bytes_per_line() - 1);
+		
 		return std::min(
 			d_offset + BitOffset((up_base + (up_row * max_bytes_per_line()) + column), 0),
 			(d_offset + d_length - 1));
@@ -1140,6 +1152,8 @@ REHex::DocumentCtrl::Rect REHex::DisassemblyRegion::calc_offset_bounds(BitOffset
 	assert(offset >= d_offset);
 	assert(offset <= (d_offset + d_length));
 	
+	const FontCharacterCache &fcc = doc_ctrl->get_fcc();
+	
 	off_t rel_offset = (offset - d_offset).byte();
 	
 	off_t up_off = unprocessed_offset_rel();
@@ -1165,13 +1179,13 @@ REHex::DocumentCtrl::Rect REHex::DisassemblyRegion::calc_offset_bounds(BitOffset
 		{
 			return DocumentCtrl::Rect(
 				/* Left X co-ordinate of hex byte. */
-				hex_text_x + doc_ctrl->hf_string_width((line_off * 2) + (line_off / bytes_per_group)),
+				hex_text_x + fcc.fixed_string_width((line_off * 2) + (line_off / bytes_per_group)),
 				
 				/* Line number. */
 				(y_offset + instr.second->rel_y_offset),
 				
 				/* Width of hex byte. */
-				doc_ctrl->hf_string_width(2),
+				fcc.fixed_string_width(2),
 				
 				/* Height of instruction (in lines). */
 				1);
@@ -1186,7 +1200,7 @@ REHex::DocumentCtrl::Rect REHex::DisassemblyRegion::calc_offset_bounds(BitOffset
 				(y_offset + instr.second->rel_y_offset),
 				
 				/* Width of instruction disassembly. */
-				doc_ctrl->hf_string_width(instr.second->disasm.length()),
+				fcc.fixed_string_width(instr.second->disasm.length()),
 				
 				/* Height of instruction (in lines). */
 				1);
@@ -1196,13 +1210,13 @@ REHex::DocumentCtrl::Rect REHex::DisassemblyRegion::calc_offset_bounds(BitOffset
 			
 			return DocumentCtrl::Rect(
 				/* Left X co-ordinate of ASCII character. */
-				ascii_text_x + doc_ctrl->hf_string_width(line_off),
+				ascii_text_x + fcc.fixed_string_width(line_off),
 				
 				/* Line number. */
 				(y_offset + instr.second->rel_y_offset),
 				
 				/* Width of character. */
-				doc_ctrl->hf_char_width(),
+				fcc.fixed_char_width(),
 				
 				/* Height of instruction (in lines). */
 				1);
@@ -1223,13 +1237,13 @@ REHex::DocumentCtrl::Rect REHex::DisassemblyRegion::calc_offset_bounds(BitOffset
 		{
 			return DocumentCtrl::Rect(
 				/* Left X co-ordinate of ASCII character. */
-				ascii_text_x + doc_ctrl->hf_string_width(line_off),
+				ascii_text_x + fcc.fixed_string_width(line_off),
 				
 				/* Line number. */
 				(y_offset + processed_lines + up_line),
 				
 				/* Width of character. */
-				doc_ctrl->hf_char_width(),
+				fcc.fixed_char_width(),
 				
 				/* Height of instruction (in lines). */
 				1);
@@ -1237,13 +1251,13 @@ REHex::DocumentCtrl::Rect REHex::DisassemblyRegion::calc_offset_bounds(BitOffset
 		else{
 			return DocumentCtrl::Rect(
 				/* Left X co-ordinate of hex byte. */
-				hex_text_x + doc_ctrl->hf_string_width((line_off * 2) + (line_off / bytes_per_group)),
+				hex_text_x + fcc.fixed_string_width((line_off * 2) + (line_off / bytes_per_group)),
 				
 				/* Line number. */
 				(y_offset + processed_lines + up_line),
 				
 				/* Width of hex byte. */
-				doc_ctrl->hf_string_width(2),
+				fcc.fixed_string_width(2),
 				
 				/* Height (in lines). */
 				1);
