@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2018-2023 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2018-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -218,7 +218,10 @@ namespace REHex {
 				{
 					ByteRangeTreeKey k(offset, length);
 					
-					Node *n = sorted_nodes[k] = tree.find_node(k);
+					#ifndef NDEBUG
+					Node *n =
+					#endif
+					sorted_nodes[k] = tree.find_node(k);
 					assert(n != NULL);
 				}
 				
