@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2022-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2022-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -53,6 +53,11 @@ namespace REHex
 			*/
 			std::pair<BitOffset, BitOffset> get_range() const;
 			
+			/**
+			 * @brief Check if "Whole file" is selected.
+			*/
+			bool is_whole_file() const;
+			
 			void set_whole_file();
 			void set_follow_selection();
 			
