@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2018-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2018-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -31,6 +31,7 @@
 
 #include "CharacterEncoder.hpp"
 #include "NumericTextCtrl.hpp"
+#include "profile.hpp"
 #include "search.hpp"
 #include "util.hpp"
 
@@ -164,12 +165,47 @@ void REHex::Search::not_found_notification()
 	wxMessageBox("Not found", wxMessageBoxCaptionStr, (wxOK | wxICON_INFORMATION | wxCENTRE), this);
 }
 
-void REHex::Search::limit_range(off_t range_begin, off_t range_end)
+void REHex::Search::limit_range(off_t range_begin, off_t range_end, OffsetBase fmt_base)
 {
 	assert(range_begin >= 0);
+	assert(range_end > range_begin);
 	
 	this->range_begin = range_begin;
 	this->range_end   = range_end;
+	
+	char offset_str[24];
+
+	switch(fmt_base)
+	{
+		case OFFSET_BASE_HEX:
+		{
+			snprintf(offset_str, sizeof(offset_str), "0x%llX", (long long unsigned)(range_begin));
+			range_begin_tc->SetValue(offset_str);
+
+			snprintf(offset_str, sizeof(offset_str), "0x%llX", (long long unsigned)(range_end - 1));
+			range_end_tc->SetValue(offset_str);
+
+			break;
+		}
+		
+		case OFFSET_BASE_DEC:
+		{
+			snprintf(offset_str, sizeof(offset_str), "0x%lld", (long long)(range_begin));
+			range_begin_tc->SetValue(offset_str);
+
+			snprintf(offset_str, sizeof(offset_str), "0x%lld", (long long)(range_end - 1));
+			range_end_tc->SetValue(offset_str);
+			
+			break;
+		}
+
+		default:
+			assert(false); /* Unreachable. */
+			break;
+	}
+
+	range_cb->SetValue(true);
+	enable_controls();
 }
 
 void REHex::Search::require_alignment(off_t alignment, off_t relative_to_offset)
@@ -472,7 +508,7 @@ bool REHex::Search::read_base_window_controls()
 	if(range_cb->GetValue())
 	{
 		read_off_value(&range_begin, range_begin_tc, false, "start of range");
-		read_off_value(&range_begin, range_begin_tc, false, "end of range");
+		read_off_value(&range_end, range_end_tc, false, "end of range");
 		
 		++range_end;
 	}
@@ -503,6 +539,8 @@ bool REHex::Search::read_base_window_controls()
 
 void REHex::Search::thread_main(size_t window_size, size_t compare_size)
 {
+	PROFILE_SET_THREAD_GROUP(NONE);
+	
 	while(running && match_found_at < 0)
 	{
 		off_t window_begin, window_end;
