@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2021-2022 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2021-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -56,12 +56,12 @@ class ConsoleBufferTest: public ::testing::Test
 			frame(NULL, wxID_ANY, "REHex Tests"),
 			timer(&frame, wxID_ANY)
 		{
-			frame.Bind(wxEVT_IDLE, [this](wxIdleEvent &event)
+			frame.Bind(wxEVT_IDLE, [](wxIdleEvent &event)
 			{
 				wxTheApp->ExitMainLoop();
 			});
 			
-			frame.Bind(wxEVT_TIMER, [this](wxTimerEvent &event)
+			frame.Bind(wxEVT_TIMER, [](wxTimerEvent &event)
 			{
 				wxTheApp->ExitMainLoop();
 			});
