@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2020-2021 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2020-2024 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -191,7 +191,7 @@ TEST_F(DiffWindowTest, InsertDataBeforeSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->insert_data( 25, x, 10); /* Insert before range */
@@ -201,11 +201,11 @@ TEST_F(DiffWindowTest, InsertDataBeforeSelection)
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 180) << "Selection start increased by inserting data before selection";
-	EXPECT_EQ(selection_last,  189) << "Selection end increased by inserting data before selection";
+	EXPECT_EQ(selection_first, BitOffset(180, 0)) << "Selection start increased by inserting data before selection";
+	EXPECT_EQ(selection_last,  BitOffset(189, 7)) << "Selection end increased by inserting data before selection";
 }
 
 TEST_F(DiffWindowTest, InsertDataWithinSelection)
@@ -213,7 +213,7 @@ TEST_F(DiffWindowTest, InsertDataWithinSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->insert_data(159, x, 10);
@@ -229,7 +229,7 @@ TEST_F(DiffWindowTest, InsertDataAfterSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->insert_data(160, x, 10); /* Insert within range, immediately after selection */
@@ -238,11 +238,11 @@ TEST_F(DiffWindowTest, InsertDataAfterSelection)
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 150) << "Selection start not affected by inserting data after selection";
-	EXPECT_EQ(selection_last,  159) << "Selection end not affected by inserting data after selection";
+	EXPECT_EQ(selection_first, BitOffset(150, 0)) << "Selection start not affected by inserting data after selection";
+	EXPECT_EQ(selection_last,  BitOffset(159, 7)) << "Selection end not affected by inserting data after selection";
 }
 
 TEST_F(DiffWindowTest, InsertDataBeforeFullSelection)
@@ -250,7 +250,7 @@ TEST_F(DiffWindowTest, InsertDataBeforeFullSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(100, 199);
+	doc_ctrl->set_selection_raw(BitOffset(100, 0), BitOffset(199, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->insert_data( 25, x, 10); /* Insert before range */
@@ -259,11 +259,11 @@ TEST_F(DiffWindowTest, InsertDataBeforeFullSelection)
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 120) << "Selection start increased by inserting data before selection encompassing whole range";
-	EXPECT_EQ(selection_last,  219) << "Selection end increased by inserting data before selection encompassing whole range";
+	EXPECT_EQ(selection_first, BitOffset(120, 0)) << "Selection start increased by inserting data before selection encompassing whole range";
+	EXPECT_EQ(selection_last,  BitOffset(219, 7)) << "Selection end increased by inserting data before selection encompassing whole range";
 }
 
 TEST_F(DiffWindowTest, InsertDataWithinFullSelection)
@@ -271,7 +271,7 @@ TEST_F(DiffWindowTest, InsertDataWithinFullSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(100, 199);
+	doc_ctrl->set_selection_raw(BitOffset(100, 0), BitOffset(199, 7));
 	ASSERT_TRUE(doc_ctrl->has_selection());
 	
 	unsigned char x[10] = { 0 };
@@ -288,7 +288,7 @@ TEST_F(DiffWindowTest, InsertDataAfterFullSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(100, 199);
+	doc_ctrl->set_selection_raw(BitOffset(100, 0), BitOffset(199, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->insert_data(200, x, 10); /* Insert after range/selection */
@@ -296,11 +296,11 @@ TEST_F(DiffWindowTest, InsertDataAfterFullSelection)
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 100) << "Selection start not affected by inserting data after selection encompassing whole range";
-	EXPECT_EQ(selection_last,  199) << "Selection end not affected by inserting data after selection encompassing whole range";
+	EXPECT_EQ(selection_first, BitOffset(100, 0)) << "Selection start not affected by inserting data after selection encompassing whole range";
+	EXPECT_EQ(selection_last,  BitOffset(199, 7)) << "Selection end not affected by inserting data after selection encompassing whole range";
 }
 
 TEST_F(DiffWindowTest, EraseDataBeforeRange)
@@ -481,18 +481,18 @@ TEST_F(DiffWindowTest, EraseDataBeforeSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	doc1->erase_data(140, 10);
 	
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 140) << "Selection start reduced by erasing data before selection";
-	EXPECT_EQ(selection_last,  149) << "Selection end reduced by erasing data before selection";
+	EXPECT_EQ(selection_first, BitOffset(140, 0)) << "Selection start reduced by erasing data before selection";
+	EXPECT_EQ(selection_last,  BitOffset(149, 7)) << "Selection end reduced by erasing data before selection";
 }
 
 TEST_F(DiffWindowTest, EraseDataOverlappingStartOfSelection)
@@ -500,7 +500,7 @@ TEST_F(DiffWindowTest, EraseDataOverlappingStartOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	doc1->erase_data(140, 15);
 	
@@ -515,7 +515,7 @@ TEST_F(DiffWindowTest, EraseDataAtStartOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	doc1->erase_data(150, 1);
 	
@@ -530,7 +530,7 @@ TEST_F(DiffWindowTest, EraseDataAtEndOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	doc1->erase_data(159, 1);
 	
@@ -545,7 +545,7 @@ TEST_F(DiffWindowTest, EraseDataOverlappingEndOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	doc1->erase_data(155, 15);
 	
@@ -560,18 +560,18 @@ TEST_F(DiffWindowTest, EraseDataAfterSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	doc1->erase_data(160, 10);
 	
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 150) << "Selection start not affected by erasing data after selection";
-	EXPECT_EQ(selection_last,  159) << "Selection end not affected by erasing data after selection";
+	EXPECT_EQ(selection_first, BitOffset(150, 0)) << "Selection start not affected by erasing data after selection";
+	EXPECT_EQ(selection_last,  BitOffset(159, 7)) << "Selection end not affected by erasing data after selection";
 }
 
 TEST_F(DiffWindowTest, OverwriteDataBeforeSelection)
@@ -579,7 +579,7 @@ TEST_F(DiffWindowTest, OverwriteDataBeforeSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->overwrite_data(140, x, 10);
@@ -587,11 +587,11 @@ TEST_F(DiffWindowTest, OverwriteDataBeforeSelection)
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 150) << "Selection start not affected by overwriting data before selection";
-	EXPECT_EQ(selection_last,  159) << "Selection end not affected by overwriting data before selection";
+	EXPECT_EQ(selection_first, BitOffset(150, 0)) << "Selection start not affected by overwriting data before selection";
+	EXPECT_EQ(selection_last,  BitOffset(159, 7)) << "Selection end not affected by overwriting data before selection";
 }
 
 TEST_F(DiffWindowTest, OverwriteDataOverlappingStartOfSelection)
@@ -599,7 +599,7 @@ TEST_F(DiffWindowTest, OverwriteDataOverlappingStartOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->overwrite_data(145, x, 10);
@@ -615,7 +615,7 @@ TEST_F(DiffWindowTest, OverwriteDataAtStartOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->overwrite_data(150, x, 1);
@@ -631,7 +631,7 @@ TEST_F(DiffWindowTest, OverwriteDataAtEndOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->overwrite_data(159, x, 1);
@@ -647,7 +647,7 @@ TEST_F(DiffWindowTest, OverwriteDataOverlappingEndOfSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->overwrite_data(155, x, 10);
@@ -663,7 +663,7 @@ TEST_F(DiffWindowTest, OverwriteDataAfterSelection)
 	auto range = diff_window->add_range(DiffWindow::Range(doc1, main_doc_ctrl1, 100, 100));
 	SafeWindowPointer<DocumentCtrl> doc_ctrl(range->_im_a_test_give_me_doc_ctrl());
 	
-	doc_ctrl->set_selection_raw(150, 159);
+	doc_ctrl->set_selection_raw(BitOffset(150, 0), BitOffset(159, 7));
 	
 	unsigned char x[10] = { 0 };
 	doc1->overwrite_data(160, x, 10);
@@ -671,9 +671,9 @@ TEST_F(DiffWindowTest, OverwriteDataAfterSelection)
 	auto ranges = diff_window->get_ranges();
 	ASSERT_EQ(ranges.size(), 1U);
 	
-	off_t selection_first, selection_last;
+	BitOffset selection_first, selection_last;
 	std::tie(selection_first, selection_last) = doc_ctrl->get_selection_raw();
 	
-	EXPECT_EQ(selection_first, 150) << "Selection start not affected by overwriting data after selection";
-	EXPECT_EQ(selection_last,  159) << "Selection end not affected by overwriting data after selection";
+	EXPECT_EQ(selection_first, BitOffset(150, 0)) << "Selection start not affected by overwriting data after selection";
+	EXPECT_EQ(selection_last,  BitOffset(159, 7)) << "Selection end not affected by overwriting data after selection";
 }
