@@ -384,8 +384,11 @@
 #ifndef wxLUA_USEBINDING_WXNET
     #define wxLUA_USEBINDING_WXNET                      1
 #endif
+#ifndef wxLUA_USEBINDING_WXPROPGRID
+    #define wxLUA_USEBINDING_WXPROPGRID                 1
+#endif
 #ifndef wxLUA_USEBINDING_WXRICHTEXT
-    #define wxLUA_USEBINDING_WXRICHTEXT                 0 // NOT FINISHED or WORKING
+    #define wxLUA_USEBINDING_WXRICHTEXT                 1 // NOT FINISHED or WORKING
 #endif
 #ifndef wxLUA_USEBINDING_WXSTC
     #define wxLUA_USEBINDING_WXSTC                      1
@@ -500,6 +503,14 @@
     #define WXLUA_IMPLEMENT_BIND_WXNET
 #endif // wxLUA_USEBINDING_WXNET
 
+#if wxLUA_USEBINDING_WXPROPGRID
+    #define WXLUA_DECLARE_BIND_WXPROPGRID  extern WXDLLIMPEXP_BINDWXPROPGRID  wxLuaBinding* wxLuaBinding_wxpropgrid_init(); // modules/wxbind
+    #define WXLUA_IMPLEMENT_BIND_WXPROPGRID  wxLuaBinding_wxpropgrid_init();
+#else
+    #define WXLUA_DECLARE_BIND_WXPROPGRID
+    #define WXLUA_IMPLEMENT_BIND_WXPROPGRID
+#endif // wxLUA_USEBINDING_WXPROPGRID
+
 #if wxLUA_USEBINDING_WXRICHTEXT && wxUSE_RICHTEXT
     #define WXLUA_DECLARE_BIND_WXRICHTEXT extern WXDLLIMPEXP_BINDWXRICHTEXT wxLuaBinding* wxLuaBinding_wxrichtext_init(); // modules/wxbind
     #define WXLUA_IMPLEMENT_BIND_WXRICHTEXT wxLuaBinding_wxrichtext_init();
@@ -557,6 +568,7 @@
     WXLUA_DECLARE_BIND_WXHTML \
     WXLUA_DECLARE_BIND_WXAUI \
     WXLUA_DECLARE_BIND_WXMEDIA \
+    WXLUA_DECLARE_BIND_WXPROPGRID \
     WXLUA_DECLARE_BIND_WXRICHTEXT \
     WXLUA_DECLARE_BIND_WXWEBVIEW \
     WXLUA_DECLARE_BIND_WXGL \
@@ -586,6 +598,7 @@
     WXLUA_IMPLEMENT_BIND_WXHTML \
     WXLUA_IMPLEMENT_BIND_WXAUI \
     WXLUA_IMPLEMENT_BIND_WXMEDIA \
+    WXLUA_IMPLEMENT_BIND_WXPROPGRID \
     WXLUA_IMPLEMENT_BIND_WXRICHTEXT \
     WXLUA_IMPLEMENT_BIND_WXWEBVIEW \
     WXLUA_IMPLEMENT_BIND_WXGL \
